package cz.cvut.kbss.wpa.jpa;

import cz.cvut.kbss.wpa.jpa.datagen.EmployeeGenerator;
import cz.cvut.kbss.wpa.jpa.model.Course;
import cz.cvut.kbss.wpa.jpa.model.Department;
import cz.cvut.kbss.wpa.jpa.model.Employee;
import cz.cvut.kbss.wpa.jpa.model.Student;
import java.util.List;
import javax.persistence.EntityManager;

public class Main {

    public static void main(String[] args) {
        new EmployeeGenerator().generateEmployees(100);
        final EntityManager em = PersistenceFactory.createNewEntityManager();
        try {
            listAllEmployees(em);

            addEmployeeToDepartment(em);

            addStudentsAndCourses(em);
        } finally {
            em.close();
        }
    }

    private static void listAllEmployees(EntityManager em) {
        final List<Employee> employees = em.createQuery("SELECT e FROM Employee e", Employee.class).getResultList();
        System.out.println("---------------------------");
        System.out.println("We have " + employees.size() + " employees:");
        employees.forEach(e -> System.out.println(e));
        System.out.println("---------------------------");
    }

    /**
     * Exercises the bidirectional OneToMany - ManyToOne relationship between
     * Employee and Department.
     *
     * @param em
     */
    private static void addEmployeeToDepartment(EntityManager em) {
        final Employee catherine = new Employee("Catherine", "Halsey");
        final Department dept = new Department("ONI");
        dept.addEmployee(catherine);
        em.getTransaction().begin();
        em.persist(dept);
        em.persist(catherine);
        em.getTransaction().commit();

        final Department result = em.find(Department.class, dept.getId());
        System.out.println("---------------------------");
        System.out.println("Department " + result + " has " + result.getEmployees().size() + " employee(s): ");
        System.out.println(result.getEmployees());
        System.out.println("---------------------------");
    }

    private static void addStudentsAndCourses(EntityManager em) {
        final Student flynn = new Student("Flynn", "Carsen");
        flynn.setEmail("thelibrarian@museum.org");

        final Course wpa = new Course("Webove a podnikove Aplikace");
        em.getTransaction().begin();
        em.persist(flynn);
        em.persist(wpa);
        flynn.enrollInCourse(wpa);
        em.getTransaction().commit();

        // Gets all students which are taking the specified course
        // Notice how we are traversing the relationship
        // Also notice usage of parameters in the query (delimited by :)
        final List<Student> students = em.createQuery("SELECT s FROM Student s WHERE s.courseTaken = :course", Student.class).setParameter("course", wpa).getResultList();
        assert !students.isEmpty();
        System.out.println("---------------------------");
        System.out.println("List of students taking course " + wpa);
        System.out.println(students);
        System.out.println("---------------------------");
    }
}
