package cz.cvut.kbss.wpa.jpa.model;

import java.util.ArrayList;
import java.util.List;
import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;

@Entity
public class Department extends BaseEntity {

    @Basic(optional = false)
    @Column(name = "department", unique = true, nullable = false)
    private String name;

    // Employees are fetched lazily - only when asked for
    @OneToMany(mappedBy = "department", fetch = FetchType.LAZY)
    @OrderBy("surname, name")   // Orders the employees first by surname, then by name
    private List<Employee> employees;

    public Department() {
    }

    public Department(String name) {
        this.name = name;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public void setEmployees(List<Employee> employees) {
        this.employees = employees;
    }

    public List<Employee> getEmployees() {
        return employees;
    }

    /**
     * Adds employee to this department.
     *
     * Notice that this method sets both sides of the relationship. Quite
     * convenient.
     *
     * It is also more logical to add an employee to a department rather then
     * set a department on an employee.
     *
     * @param e The employee to add
     */
    public void addEmployee(Employee e) {
        assert e != null;
        if (employees == null) {
            this.employees = new ArrayList<>();
        }
        employees.add(e);
        e.setDepartment(this);
    }

    @Override
    public String toString() {
        return name;
    }
}
