package cz.cvut.kbss.wpa.jpa.model;

import java.util.ArrayList;
import java.util.List;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;

@Entity
public class Student extends Person {

    private String email;

    // We are explicitly defining the join table.
    // This is not necessary, the join table would be created by the provider anyway. This way,
    // we are just specifying names of the join columns. Try removing the @JoinTable annotation and running the app again.
    @ManyToMany(fetch = FetchType.EAGER)    // Eager fetching for courses
    @JoinTable(joinColumns = {
        @JoinColumn(name = "COURSE_ID")}, inverseJoinColumns = {
        @JoinColumn(name = "STUDENT_ID")})
    private List<Course> courseTaken;

    public Student() {
    }

    public Student(String name, String surname) {
        super(name, surname);
    }

    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    public List<Course> getCourseTaken() {
        // This is a common pattern of preventing checking for null collections in the application itself
        if (courseTaken == null) {
            this.courseTaken = new ArrayList<>();
        }
        return courseTaken;
    }

    public void setCourseTaken(List<Course> courseTaken) {
        this.courseTaken = courseTaken;
    }

    public void enrollInCourse(Course course) {
        getCourseTaken().add(course);
    }

    @Override
    public String toString() {
        return "Student{" + super.toString() + ", email=" + email + '}';
    }
}
